//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.4_testing/src/org/deegree/ogcwebservices/wmps/operation/GetAvailableTemplates.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
 - Department of Geography, University of Bonn -
 and
 - lat/lon GmbH -

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
 ----------------------------------------------------------------------------*/
package org.deegree.ogcwebservices.wmps.operation;

import java.util.HashMap;
import java.util.Map;

import org.deegree.framework.xml.XMLParsingException;
import org.deegree.framework.xml.XMLTools;
import org.w3c.dom.Element;

/**
 * Can be requested as KVP:
 * <p>
 * ...?request=GetAvailableTemplates&version=1.0.0
 * </p>
 * or as XML:
 * <p>
 * 
 * <pre>
 * &lt;?xml version=&quot;1.0&quot; encoding=&quot;UTF-8&quot;?&gt;
 * &lt;deegreewmps:GetAvailableTemplates xmlns:deegreewmps=&quot;http://www.deegree.org/wmps&quot; version=&quot;1.0.0&quot;/&gt;
 * </pre>
 * 
 * </p>
 * 
 * @author <a href="mailto:name@deegree.org">Andreas Poth</a>
 * @author last edited by: $Author: apoth $
 * 
 * @version $Revision: 20967 $, $Date: 2009-11-23 16:49:00 +0100 (Mo, 23. Nov 2009) $
 */
public class GetAvailableTemplates extends WMPSRequestBase {

    private static final long serialVersionUID = 1210026775322614277L;

    /**
     * @param id
     * @param version
     * @param vendorSpecificParameter
     */
    public GetAvailableTemplates( String id, String version, Map<String, String> vendorSpecificParameter ) {
        super( version, id, vendorSpecificParameter );
    }

    /**
     * 
     * @param parameter
     * @return new {@link GetAvailableTemplates}
     */
    public static GetAvailableTemplates create( Map<String, String> parameter ) {
        String id = "" + System.currentTimeMillis();
        String version = retrieveVersionParameter( parameter );
        return new GetAvailableTemplates( id, version, parameter );
    }

    /**
     * 
     * @param request
     * @return new {@link GetAvailableTemplates}
     * @throws XMLParsingException
     */
    public static GetAvailableTemplates create( Element request )
                            throws XMLParsingException {
        String id = "" + System.currentTimeMillis();
        String version;
        try {
            version = XMLTools.getRequiredAttrValue( "version", null, request );
        } catch ( XMLParsingException e ) {
            throw new XMLParsingException( "Error parsing required attribute parameter 'Version'. " + e.getMessage() );
        }
        return new GetAvailableTemplates( id, version, new HashMap<String, String>() );
    }

    /**
     * Parse 'Version' Parameter.
     * 
     * @param model
     * @return String version (default=1.0.0)
     */
    private static String retrieveVersionParameter( Map<String, String> model ) {

        String version = null;
        if ( model.containsKey( "VERSION" ) ) {
            version = (String) model.remove( "VERSION" );
        }
        if ( version == null ) {
            /** default value set as per the WMPS draft specifications. */
            version = "1.0.0";
        }

        return version;
    }

}
